import { getBodyRegex, getCustomVarRegex, getTabstopsRecordRegex, getTabstopsRegex } from "./regex";
export function replaceBody(code, options) {
  if (!(options !== null && options !== void 0 && options.bodyRecord)) {
    return code;
  }

  const reg = getBodyRegex();
  return code.replaceAll(reg, function (match, snippetName, bodyStartIdx, comma, bodyEndIdx) {
    var _options$bodyRecord;

    const body = options === null || options === void 0 ? void 0 : (_options$bodyRecord = options.bodyRecord) === null || _options$bodyRecord === void 0 ? void 0 : _options$bodyRecord[snippetName];

    if (!body) {
      return match;
    }

    if (!comma && !bodyEndIdx) {
      return body[Number(bodyStartIdx)];
    }

    if (bodyEndIdx) {
      return body.slice(Number(bodyStartIdx), Number(bodyEndIdx)).join('\r');
    }

    if (comma) {
      return body.slice(Number(bodyStartIdx)).join('\r');
    }

    return match;
  });
}
export function replaceTabstops(code, snippetName, options) {
  if (!(options !== null && options !== void 0 && options.tabstopsRecord)) {
    return code;
  }

  const reg = getTabstopsRegex();
  return code.replaceAll(reg, function (match) {
    var _options$tabstopsReco, _options$tabstopsReco2;

    const newTabstops = (_options$tabstopsReco = options.tabstopsRecord) === null || _options$tabstopsReco === void 0 ? void 0 : (_options$tabstopsReco2 = _options$tabstopsReco[snippetName]) === null || _options$tabstopsReco2 === void 0 ? void 0 : _options$tabstopsReco2[Number(match)];

    if (newTabstops) {
      return String(newTabstops);
    }

    return match;
  });
}
export function getTabstopsRecord(tabstops) {
  const tabstopsRecord = {};
  const reg = getTabstopsRecordRegex();

  for (const [newTabstops, snippetTabstops] of Object.entries(tabstops)) {
    for (const snippetTabstop of snippetTabstops) {
      const matches = reg.exec(snippetTabstop);

      if (!matches) {
        throw Error(`Match ${snippetTabstop} fail`);
      }

      const snippetName = matches[1];
      const tabstops = Number(matches[2]);

      if (!tabstopsRecord[snippetName]) {
        tabstopsRecord[snippetName] = {};
      }

      tabstopsRecord[snippetName][tabstops] = Number(newTabstops);
    }
  }

  return tabstopsRecord;
}
export function replaceCustomVar(code, options) {
  if (!(options !== null && options !== void 0 && options.customVarRecord)) {
    return code;
  }

  const reg = getCustomVarRegex();
  return code.replaceAll(reg, function (match, customVarName) {
    var _options$customVarRec;

    const res = options === null || options === void 0 ? void 0 : (_options$customVarRec = options.customVarRecord) === null || _options$customVarRec === void 0 ? void 0 : _options$customVarRec[customVarName];

    if (typeof res === 'function') {
      return res();
    }

    if (typeof res === 'string') {
      return res;
    }

    return match;
  });
}