import _createForOfIteratorHelper from "@babel/runtime/helpers/esm/createForOfIteratorHelper";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import { getBodyRegex, getCustomVarRegex, getTabstopsRecordRegex, getTabstopsRegex } from "./regex";
export function replaceBody(code, options) {
  if (!(options !== null && options !== void 0 && options.bodyRecord)) {
    return code;
  }

  var reg = getBodyRegex();
  return code.replaceAll(reg, function (match, snippetName, bodyStartIdx, comma, bodyEndIdx) {
    var _options$bodyRecord;

    var body = options === null || options === void 0 ? void 0 : (_options$bodyRecord = options.bodyRecord) === null || _options$bodyRecord === void 0 ? void 0 : _options$bodyRecord[snippetName];

    if (!body) {
      return match;
    }

    if (!comma && !bodyEndIdx) {
      return body[Number(bodyStartIdx)];
    }

    if (bodyEndIdx) {
      return body.slice(Number(bodyStartIdx), Number(bodyEndIdx)).join('\r');
    }

    if (comma) {
      return body.slice(Number(bodyStartIdx)).join('\r');
    }

    return match;
  });
}
export function replaceTabstops(code, snippetName, options) {
  if (!(options !== null && options !== void 0 && options.tabstopsRecord)) {
    return code;
  }

  var reg = getTabstopsRegex();
  return code.replaceAll(reg, function (match) {
    var _options$tabstopsReco, _options$tabstopsReco2;

    var newTabstops = (_options$tabstopsReco = options.tabstopsRecord) === null || _options$tabstopsReco === void 0 ? void 0 : (_options$tabstopsReco2 = _options$tabstopsReco[snippetName]) === null || _options$tabstopsReco2 === void 0 ? void 0 : _options$tabstopsReco2[Number(match)];

    if (newTabstops) {
      return String(newTabstops);
    }

    return match;
  });
}
export function getTabstopsRecord(tabstops) {
  var tabstopsRecord = {};
  var reg = getTabstopsRecordRegex();

  for (var _i = 0, _Object$entries = Object.entries(tabstops); _i < _Object$entries.length; _i++) {
    var _Object$entries$_i = _slicedToArray(_Object$entries[_i], 2),
        newTabstops = _Object$entries$_i[0],
        snippetTabstops = _Object$entries$_i[1];

    var _iterator = _createForOfIteratorHelper(snippetTabstops),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var snippetTabstop = _step.value;
        var matches = reg.exec(snippetTabstop);

        if (!matches) {
          throw Error("Match ".concat(snippetTabstop, " fail"));
        }

        var snippetName = matches[1];

        var _tabstops = Number(matches[2]);

        if (!tabstopsRecord[snippetName]) {
          tabstopsRecord[snippetName] = {};
        }

        tabstopsRecord[snippetName][_tabstops] = Number(newTabstops);
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  }

  return tabstopsRecord;
}
export function replaceCustomVar(code, options) {
  if (!(options !== null && options !== void 0 && options.customVarRecord)) {
    return code;
  }

  var reg = getCustomVarRegex();
  return code.replaceAll(reg, function (match, customVarName) {
    var _options$customVarRec;

    var res = options === null || options === void 0 ? void 0 : (_options$customVarRec = options.customVarRecord) === null || _options$customVarRec === void 0 ? void 0 : _options$customVarRec[customVarName];

    if (typeof res === 'function') {
      return res();
    }

    if (typeof res === 'string') {
      return res;
    }

    return match;
  });
}